##################################################################
#  Oracle RDS - Parameter and Option Groups
##################################################################
locals {
  enable_oem_option = var.major_engine_version19 == "19" || var.major_engine_version == "19" ? [{ option_name : "OEM", port : var.port_oem, vpc_security_group_memberships : var.security_group_id }] : []
  enable_oem_option19 = [{ option_name : "OEM", port : var.port_oem, vpc_security_group_memberships : var.security_group_id }]
  oem_agent_version   = "13.5.0.0.v1"

  required_db_parameters = [
      {
        name         = "audit_trail"
        value        = "XML,EXTENDED"
        apply_method = "pending-reboot"
      },
      {
        name         = "control_management_pack_access"
        value        = "DIAGNOSTIC+TUNING"
        apply_method = "immediate"
      },
      {
        name         = "control_file_record_keep_time"
        value        = "31"
        apply_method = "immediate"
      },
      {
        name         = "db_block_checksum"
        value        = "FULL"
        apply_method = "immediate"
      },
      {
        name         = "db_block_checking"
        value        = "FULL"
        apply_method = "immediate"
      },
      {
        name         = "db_flashback_retention_target"
        value        = "120"
        apply_method = "immediate"
      },
      {
        name         = "db_lost_write_protect"
        value        = "TYPICAL"
        apply_method = "immediate"
      },
      {
        name         = "db_ultra_safe"
        value        = "OFF"
        apply_method = "pending-reboot"
      },
      {
        name         = "fast_start_mttr_target"
        value        = "300"
        apply_method = "immediate"
      },
      {
        name         = "global_names"
        value        = "TRUE"
        apply_method = "immediate"
      },
      {
        name         = "log_buffer"
        value        = "16777216"
        apply_method = "pending-reboot"
      },
      {
        name         = "max_dump_file_size"
        value        = "10M"
        apply_method = "immediate"
      },
      {
        name         = "optimizer_mode"
        value        = "ALL_ROWS"
        apply_method = "immediate"
      },
      {
        name         = "parallel_execution_message_size"
        value        = "32768"
        apply_method = "pending-reboot"
      },
      {
        name         = "statistics_level"
        value        = "TYPICAL"
        apply_method = "immediate"
      },
      {
        name         = "undo_retention"
        value        = "86400"
        apply_method = "immediate"
      },
      {
        name         = "sql92_security"
        value        = "TRUE"
        apply_method = "pending-reboot"
      },
      {
        name         = "resource_limit"
        value        = "TRUE"
        apply_method = "immediate"
      },
      {
        name         = "sec_max_failed_login_attempts"
        value        = "5"
        apply_method = "pending-reboot"
      },
      {
        name         = "sec_protocol_error_further_action"
        value        = "(DELAY,30)"
        apply_method = "immediate"
      },
      {
        name         = "sec_protocol_error_trace_action"
        value        = "LOG"
        apply_method = "immediate"
      },
      {
        name         = "memory_target"
        value        = "0"
        apply_method = "immediate"
      },
      {
        name         = "memory_max_target"
        value        = "0"
        apply_method = "pending-reboot"
      },
      {
        name         = "pga_aggregate_target"
        #value       = "{DBInstanceClassMemory*1/8}"
        value        =  var.custom_pga == null ? "{DBInstanceClassMemory*1/8}" : var.custom_pga
        apply_method = "immediate"
      },
      {
        name         = "sga_target"
        #value        = "{DBInstanceClassMemory*3/4}"
        value        = var.custom_sga == null ? "{DBInstanceClassMemory*3/4}" : var.custom_sga
        apply_method = "immediate"
      },
      {
        name         = "sga_max_size"
        value        = var.custom_sga == null ? "{DBInstanceClassMemory*3/4}" : var.custom_sga
        apply_method = "pending-reboot"
      },
      {
        name         = "db_16k_cache_size"
        value        = var.db_16k_cache_size == null ? 0 : var.db_16k_cache_size
        apply_method = "immediate"
      },
      {
        name         = "db_32k_cache_size"
        value        = var.db_32k_cache_size == null ? 0 : var.db_32k_cache_size
        apply_method = "immediate"
      },
      {
        name         = "use_large_pages"
        value        = "ONLY"
        apply_method = "pending-reboot"
      },
      {
        name         = "enable_goldengate_replication"
        value        = upper(var.goldengate_replication.enable_goldengate_replication)
        apply_method = "immediate"
      },
      {
        name         = "streams_pool_size"
        value        = "1280000000"
        apply_method = "immediate"
      },
      {
        name         = "db_files"
        value        = "2048"
        apply_method = "pending-reboot"
      },
      {
        name         = "optimizer_secure_view_merging"
        value        = "TRUE"
        apply_method = "immediate"
      },
      {
        name         = "sec_return_server_release_banner"
        value        = "FALSE"
        apply_method = "pending-reboot"
      },
     
    ]
  all_db_parameters      = concat(local.required_db_parameters, var.custom_db_parameters)
}

resource "aws_db_parameter_group" "default19" {
  name        = "${var.db_identifier_name}-parameter-group19"
  description = format("Parameter group 19 for %s", var.db_identifier_name)
  family      = var.parameter_group_family19

  dynamic "parameter" {
    for_each = local.all_db_parameters
    content {
      name         = parameter.value.name
      value        = parameter.value.value
      apply_method = lookup(parameter.value, "apply_method", null)
    }
  }
  dynamic "parameter" {
    for_each = var.parameter_fix_control_value != null ? [1] : []
    content {
      name         = "_fix_control"
      value        = var.parameter_fix_control_value
      apply_method = "immediate"
    }
  }
  tags = var.tags

  lifecycle {
    create_before_destroy = true
  }
}
# Resource section for db parameter groups
resource "aws_db_parameter_group" "default" {
  name        = "${var.db_identifier_name}-parameter-group"
  description = format("Parameter group for %s", var.db_identifier_name)
  family      = var.parameter_group_family

  dynamic "parameter" {
    for_each = local.all_db_parameters
    content {
      name         = parameter.value.name
      value        = parameter.value.value
      apply_method = lookup(parameter.value, "apply_method", null)
    }
  }
  dynamic "parameter" {
    for_each = var.parameter_fix_control_value != null ? [1] : []
    content {
      name         = "_fix_control"
      value        = var.parameter_fix_control_value
      apply_method = "immediate"
    }
  }
  tags = var.tags

  lifecycle {
    create_before_destroy = true
  }
}

# Resource section for db option group
resource "aws_db_option_group" "default" {
  name                     = "${var.db_identifier_name}-option-group"
  option_group_description = format("Option group for %s", var.db_identifier_name)
  engine_name              = var.engine_name
  major_engine_version     = var.major_engine_version
  tags                     = var.tags

  option {
    option_name = "NATIVE_NETWORK_ENCRYPTION"

    option_settings {
      name  = "SQLNET.ENCRYPTION_SERVER"
      value = "REQUIRED"
    }
    option_settings {
      name  = "SQLNET.ENCRYPTION_TYPES_SERVER"
      value = "AES256"
    }
    option_settings {
      name  = "SQLNET.CRYPTO_CHECKSUM_SERVER"
      value = "REQUIRED"
    }
    option_settings {
      name  = "SQLNET.CRYPTO_CHECKSUM_TYPES_SERVER"
      value = "SHA512,SHA384,SHA256"
    }
    option_settings {
      name  = "SQLNET.CRYPTO_CHECKSUM_TYPES_CLIENT"
      value = "SHA512,SHA384,SHA256"
    }
    option_settings {
      name  = "SQLNET.ENCRYPTION_CLIENT"
      value = "REQUIRED"
    }
    option_settings {
      name  = "SQLNET.CRYPTO_CHECKSUM_CLIENT"
      value = "REQUIRED"
    }
    option_settings {
      name  = "SQLNET.ENCRYPTION_TYPES_CLIENT"
      value = "AES256"
    }
  }

  option {
    option_name = "S3_INTEGRATION"
    version = "1.0"
  }

  option {
    option_name = "TIMEZONE_FILE_AUTOUPGRADE"
  }

  option {
    option_name = "SQLT"
    version = "2018-07-25.v1"
    option_settings {
      name  = "LICENSE_PACK"
      value = "T"
    }
  }

  dynamic "option" {
    for_each = var.timezone != null ? [1] : []
    content {
      option_name = "Timezone"
      option_settings {
          name  = "TIME_ZONE"
          value = var.timezone
        }
      }
  }

  dynamic "option" {
    for_each = var.oem_agent == true ? [1] : []
      content {
        option_name = "OEM_AGENT"
  	    version = local.oem_agent_version
  	    port = "3872"
  	    vpc_security_group_memberships = var.security_group_id

        option_settings {
          name  = "MINIMUM_TLS_VERSION"
          value = "TLSv1.2"
        }
  	    option_settings {
          name  = "TLS_CIPHER_SUITE"
          value = "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384"
         }
  	    option_settings {
          name  = "AGENT_REGISTRATION_PASSWORD"
          value = var.oem_agent_rgs_key
         }
  	    option_settings {
          name  = "OMS_HOST"
          value = "oem-na.abcd.net"
        }
  	    option_settings {
          name  = "OMS_PORT"
          value = "4903"
         }
      }
  }

  dynamic "option" {
    for_each = var.oem_agent == true ? [] : [1]
    content {
      option_name = "OEM"
      port = var.port_oem
      vpc_security_group_memberships = var.security_group_id
     }
  }

  lifecycle {
    create_before_destroy = true
  }

}
resource "aws_db_option_group" "default19" {
  name                     = "${var.db_identifier_name}-option-group19"
  option_group_description = format("Option group for %s", var.db_identifier_name)
  engine_name              = var.engine_name
  major_engine_version     = var.major_engine_version19
  tags                     = var.tags

  option {
    option_name = "NATIVE_NETWORK_ENCRYPTION"

    option_settings {
      name  = "SQLNET.ENCRYPTION_SERVER"
      value = "REQUIRED"
    }
    option_settings {
      name  = "SQLNET.ENCRYPTION_TYPES_SERVER"
      value = "AES256"
    }
    option_settings {
      name  = "SQLNET.CRYPTO_CHECKSUM_SERVER"
      value = "REQUIRED"
    }
    option_settings {
      name  = "SQLNET.CRYPTO_CHECKSUM_TYPES_SERVER"
      value = "SHA512,SHA384,SHA256"
    }
    option_settings {
      name  = "SQLNET.CRYPTO_CHECKSUM_TYPES_CLIENT"
      value = "SHA512,SHA384,SHA256"
    }
    option_settings {
      name  = "SQLNET.ENCRYPTION_CLIENT"
      value = "REQUIRED"
    }
    option_settings {
      name  = "SQLNET.CRYPTO_CHECKSUM_CLIENT"
      value = "REQUIRED"
    }
    option_settings {
      name  = "SQLNET.ENCRYPTION_TYPES_CLIENT"
      value = "AES256"
    }
  }

  option {
    option_name = "S3_INTEGRATION"
    version = "1.0"
  }

  option {
    option_name = "TIMEZONE_FILE_AUTOUPGRADE"
  }

  option {
    option_name = "SQLT"
    version = "2018-07-25.v1"
    option_settings {
      name  = "LICENSE_PACK"
      value = "T"
    }
  }


  dynamic "option" {
    for_each = var.timezone != null ? [1] : []
    content {
      option_name = "Timezone"
      option_settings {
          name  = "TIME_ZONE"
          value = var.timezone
        }
      }
  }

  dynamic "option" {
    for_each = var.oem_agent == true ? [1] : []
      content {
        option_name = "OEM_AGENT"
  	    version = local.oem_agent_version
  	    port = "3872"
  	    vpc_security_group_memberships = var.security_group_id

        option_settings {
          name  = "MINIMUM_TLS_VERSION"
          value = "TLSv1.2"
        }
  	    option_settings {
          name  = "TLS_CIPHER_SUITE"
          value = "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384"
         }
  	    option_settings {
          name  = "AGENT_REGISTRATION_PASSWORD"
          value = var.oem_agent_rgs_key
         }
  	    option_settings {
          name  = "OMS_HOST"
          value = "oem-na.abcd.net"
        }
  	    option_settings {
          name  = "OMS_PORT"
          value = "4903"
         }
      }
  }

  dynamic "option" {
    for_each = var.oem_agent == true ? [] : [1]
    content {
      option_name = "OEM"
      port = var.port_oem
      vpc_security_group_memberships = var.security_group_id
     }
  }

  lifecycle {
    create_before_destroy = true
  }

}

