##############
# Networking #
##############
data "aws_subnets" "subnet_ids" {
  dynamic filter {
    for_each = length(var.subnets) == 0 ? [""] : []
    content {
      name = "tag:routable"
      # This value should *MATCH* the previous default used by the Module, prior to replacing with this code.  This is to support backwards compatibility.
      values = ["false"]
    }
  }
  dynamic filter {
    for_each = length(var.subnets) == 0 ? [] : [""]
    content {
      name = "tag:Name"
      values = [var.subnets]
    }
  }
}

data "aws_subnet" "subnets" {
  for_each = toset(data.aws_subnets.subnet_ids.ids)
  id       = each.value
}

data "aws_route_table" "public_subnet_based_on_cidr" {
  for_each = data.aws_subnet.subnets
  filter {
    name   = "route.destination-cidr-block"
    values = ["0.0.0.0/0"]
  }

  filter {
    name   = "route.transit-gateway-id"
    values = ["tgw-*"]
  }
}

data "aws_subnet" "private_subnet_ids" {
  count = length(local.private_subnets)
  filter {
    name   = "tag:Name"
    values = [sort(local.private_subnets)[count.index]]
  }
}