######################
# Primary DB Outputs #
######################
output "database_arn" {
  description = "The Oracle Database ARN"
  value       = aws_db_instance.rds.arn
}

output "database_host" {
  description = "The Oracle Database Host"
  value       = aws_db_instance.rds.address
}

output "database_port" {
  description = "The Port of the Oracle Database"
  value       = aws_db_instance.rds.port
}

output "database_endpoint" {
  description = "The Endpoint name of the Oracle Database"
  value       = aws_db_instance.rds.endpoint
}

output "database_name" {
  description = "The Name of the Oracle Database"
  value       = aws_db_instance.rds.db_name
}

output "jdbc_connection_string" {
  description = "JDBC connection string for the Oracle Database"
  value       = "jdbc:oracle:thin:@//${aws_db_instance.rds.endpoint}/${aws_db_instance.rds.db_name}"
}

output "database_identifier" {
  description = "The Instance ID of the Oracle Database"
  value       = aws_db_instance.rds.identifier
}

output "database_engine" {
  description = "The Database Engine Used"
  value       = aws_db_instance.rds.engine
}

output "database_engine_version" {
  description = "The Database Engine Version"
  value       = aws_db_instance.rds.engine_version
}

output "rds_oracle_customer_managed_security_group_id" {
  description = "Empty Oracle security group for users to use to attach security group(s) from other services which needs to communicate with RDS cluster."
  value       = aws_security_group.rds_oracle_customer_managed_security_group.id
}

output "database_subnets_ids" {
  description = "List of subnets ids attached to Oracle RDS"
  value       = module.rds-oracle-subnet-groups.subnet_ids
}

output "s3_integration_role_name" {
  description = "S3 integration role name"
  value       = aws_iam_role.s3_integration.name
}
output "s3_integration_role_arn" {
  description = "S3 integration role arn"
  value       = aws_db_instance_role_association.oracle_rds_s3_integration.role_arn
}

output "db_kms_key_arn" {
  value = aws_db_instance.rds.kms_key_id
}

output "db_maintenance_window" {
  description = "The DB maintenance window"
  value       = aws_db_instance.rds.maintenance_window
}

output "db_instance_ca_cert_identifier" {
  description = "Specifies the identifier of the CA certificate for the DB instance"
  value       = aws_db_instance.rds.ca_cert_identifier
}
